'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _gsap = require('gsap');

var _CountdownItem = {
  'container': 'CountdownItem__container___30Ymr',
  'backBottom': 'CountdownItem__backBottom___2AGBz',
  'backTop': 'CountdownItem__backTop___1t8Sv',
  'front': 'CountdownItem__front___2NqDz',
  'frontInverted': 'CountdownItem__frontInverted___3wDsB'
};

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var toPx = function toPx(x) {
  return x + 'px';
};
var fontSize = function fontSize(size) {
  return toPx(size * 0.8);
};

var styleGenerator = function styleGenerator(_ref) {
  var _ref$size = _ref.size,
      size = _ref$size === undefined ? 40 : _ref$size,
      _ref$background = _ref.background,
      background = _ref$background === undefined ? 'white' : _ref$background;
  return {
    container: {
      background: background,
      width: fontSize(size),
      height: toPx(size),
      fontSize: fontSize(size),
      lineHeight: toPx(size),
      borderRadius: toPx(fontSize / 6)
    }
  };
};

exports.default = {
  name: 'CountdownItem',
  props: {
    value: { type: Number, default: 0 },
    size: { type: Number, default: 40 },
    background: { type: String, default: 'white' }
  },
  data: function data() {
    return {
      cachedValue: this.value,
      inverted: false,
      hidden: true
    };
  },


  methods: {
    rotate: function rotate(rotationX) {
      var frontEl = this.$refs.frontEl;

      return new _promise2.default(function (resolve) {
        _gsap.TweenLite.to(frontEl, 0.4, {
          rotationX: rotationX,
          transformOrigin: 'left bottom',
          transformPerspective: 300,
          onComplete: resolve
        });
      });
    },
    turn: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var frontEl;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                frontEl = this.$refs.frontEl;

                this.hidden = false;
                _context.next = 4;
                return this.rotate('90deg');

              case 4:
                this.inverted = true;
                _context.next = 7;
                return this.rotate('180deg');

              case 7:
                this.inverted = false;
                this.cachedValue = this.value;
                this.hidden = true;
                this.$nextTick(function () {
                  return _gsap.TweenLite.set(frontEl, { rotationX: 0 });
                });

              case 11:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function turn() {
        return _ref2.apply(this, arguments);
      }

      return turn;
    }()
  },

  computed: {
    styles: function styles() {
      var size = this.size,
          background = this.background;

      return styleGenerator({ size: size, background: background });
    }
  },

  watch: {
    value: function value() {
      this.turn();
    }
  },

  render: function render() {
    var h = arguments[0];
    var value = this.value,
        inverted = this.inverted,
        cachedValue = this.cachedValue,
        styles = this.styles,
        hidden = this.hidden,
        background = this.background;


    return h(
      'div',
      { style: styles.container, 'class': _CountdownItem.container },
      [h(
        'div',
        { 'class': _CountdownItem.backTop },
        [value]
      ), h(
        'div',
        { 'class': _CountdownItem.backBottom },
        [cachedValue]
      ), h(
        'div',
        { ref: 'frontEl', style: [{ visibility: hidden ? 'hidden' : null, background: background }], 'class': _CountdownItem.front },
        [h(
          'div',
          { 'class': inverted && _CountdownItem.frontInverted },
          [inverted ? value : cachedValue]
        )]
      )]
    );
  }
};