'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CountdownItem = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _CountdownItem = require('./CountdownItem');

var _CountdownItem2 = _interopRequireDefault(_CountdownItem);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.CountdownItem = _CountdownItem2.default;


var defaultSections = [{
  label: 'Days',
  divisor: 24 * 60 * 60 * 1000,
  mod: 100
}, {
  label: 'Hours',
  divisor: 60 * 60 * 1000,
  mod: 24
}, {
  label: 'Minutes',
  divisor: 60 * 1000,
  mod: 60
}, {
  label: 'Seconds',
  divisor: 1000,
  mod: 60
}];

var convertDiff = function convertDiff(diff, divisor, mod) {
  return parseInt(diff / divisor, 10) % mod;
};

var sectionStyles = {
  container: {
    marginRight: '6px'
  },
  label: {
    fontWeight: 'bold',
    width: '100%',
    height: '16px',
    textAlign: 'center',
    fontSize: '12px'
  },
  items: {
    marginRight: '2px'
  }
};
var Section = {
  functional: true,
  render: function render(h, _ref) {
    var _ref$props = _ref.props,
        label = _ref$props.label,
        labelStyle = _ref$props.labelStyle,
        items = _ref$props.items,
        background = _ref$props.background,
        size = _ref$props.size;
    return h(
      'div',
      { style: sectionStyles.container },
      [h(
        'div',
        { style: [sectionStyles.label, labelStyle] },
        [label]
      ), h(
        'div',
        { style: { display: 'flex' } },
        [items.map(function (item) {
          return h(_CountdownItem2.default, {
            attrs: {
              value: item,

              background: background,
              size: size
            },
            style: sectionStyles.items });
        })]
      )]
    );
  }
};


var days = function days(number) {
  return 1000 * 60 * 60 * 24 * number;
};

// eslint-disable-next-line valid-typeof
var validateExistsAndHasType = function validateExistsAndHasType(type, value) {
  return value && (typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value)) === type;
};
var sectionsValidator = function sectionsValidator(sections) {
  if (!(sections instanceof Array)) return false;
  var invalid = sections.some(function (_ref2) {
    var label = _ref2.label,
        divisor = _ref2.divisor,
        mod = _ref2.mod;
    return !validateExistsAndHasType('string', label) || !validateExistsAndHasType('number', divisor) || !validateExistsAndHasType('number', mod);
  });
  return !invalid;
};

exports.default = {
  name: 'Countdown',
  props: {
    end: { type: Number, default: Date.now() + days(20) },
    background: { type: String, default: 'white' },
    size: { type: Number, default: 40 },
    /**
     * This is an array of objects which will construct each
     * couple of numbers. Format is : { label: string, divisor: number, mod: number }[]
     * Example: [{ label: 'Hours', divisor: 60 * 60 * 1000, mod: 24 }.
     */
    sections: { validator: sectionsValidator, default: function _default() {
        return defaultSections;
      } }
  },
  data: function data() {
    return { now: Date.now() };
  },

  mounted: function mounted() {
    this.listener = setInterval(this.update.bind(this), 1000);
  },
  beforeDestroy: function beforeDestroy() {
    clearInterval(this.listener);
  },


  methods: {
    update: function update() {
      this.now = Date.now();
    }
  },

  computed: {
    ended: function ended() {
      var now = this.now,
          end = this.end,
          listener = this.listener;

      var ended = now > end;
      if (ended) {
        clearInterval(listener);
        this.$emit('ended', true);
      }
      return ended;
    },
    items: function items() {
      var now = this.now,
          end = this.end,
          ended = this.ended,
          sections = this.sections;

      if (ended) return new Array(sections.length * 2).fill(0);

      var diff = new Date(end - now);
      return sections.map(function (item) {
        return convertDiff(diff, item.divisor, item.mod);
      }).map(function (item) {
        return [parseInt(item / 10, 10), item % 10];
      }).reduce(function (acc, item) {
        return acc.concat(item);
      }, []);
    }
  },

  render: function render() {
    var h = arguments[0];
    var items = this.items,
        sections = this.sections,
        background = this.background,
        size = this.size;


    var mergedSections = sections.map(function (section, index) {
      return (0, _extends3.default)({}, section, {
        items: items.slice(2 * index, 2 * index + 2)
      });
    });
    return h(
      'div',
      { style: { display: 'flex' } },
      [mergedSections.map(function (_ref3) {
        var label = _ref3.label,
            sItems = _ref3.items;
        return h(Section, {
          attrs: {
            label: label,
            items: sItems,
            background: background,
            size: size
          }
        });
      })]
    );
  }
};